Package: crul
Title: HTTP Client
Description: A simple HTTP client, with tools for making HTTP requests,
    and mocking HTTP requests. The package is built on R6, and takes
    inspiration from Ruby's 'faraday' gem (<https://rubygems.org/gems/faraday>).
    The package name is a play on curl, the widely used command line tool
    for HTTP, and this package is built on top of the R package 'curl', an
    interface to 'libcurl' (<https://curl.se/libcurl/>).
Version: 1.6.0
License: MIT + file LICENSE
Authors@R: c(
    person("Scott", "Chamberlain", role = c("aut", "cre"),
    email = "myrmecocystus@gmail.com",
    comment = c(ORCID = "0000-0003-1444-9135"))
    )
URL: https://docs.ropensci.org/crul/, https://github.com/ropensci/crul,
        https://books.ropensci.org/http-testing/
BugReports: https://github.com/ropensci/crul/issues
Encoding: UTF-8
Language: en-US
Imports: curl (>= 3.3), R6 (>= 2.2.0), urltools (>= 1.6.0), httpcode
        (>= 0.2.0), jsonlite, mime, rlang, lifecycle
Suggests: testthat (>= 3.0.0), roxygen2 (>= 7.1.1), fauxpas (>= 0.1.0),
        webmockr (>= 2.2.0), withr, knitr, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Config/testthat/edition: 3
Config/testthat/parallel: true
X-schema.org-applicationCategory: Web
X-schema.org-keywords: http, https, API, web-services, curl, download,
        libcurl, async, mocking, caching
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2025-07-21 16:40:35 UTC; sckott
Author: Scott Chamberlain [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1444-9135>)
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2025-07-23 12:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 23:35:57 UTC; unix
