# Generated by roxygen2: do not edit by hand

S3method(field_as,default)
S3method(field_as,logical)
S3method(field_as,numeric)
S3method(ok,HttpClient)
S3method(ok,character)
S3method(ok,default)
S3method(print,asyncresponses)
export(Async)
export(AsyncQueue)
export(AsyncVaried)
export(HttpClient)
export(HttpRequest)
export(HttpResponse)
export(Paginator)
export(auth)
export(crul_settings)
export(curl_verbose)
export(handle)
export(mock)
export(ok)
export(proxy)
export(set_auth)
export(set_headers)
export(set_opts)
export(set_proxy)
export(set_verbose)
export(upload)
export(url_build)
export(url_parse)
import(rlang)
importFrom(R6,R6Class)
importFrom(curl,curl_escape)
importFrom(curl,curl_fetch_disk)
importFrom(curl,curl_fetch_memory)
importFrom(curl,curl_fetch_stream)
importFrom(curl,curl_options)
importFrom(curl,curl_version)
importFrom(curl,handle_reset)
importFrom(curl,handle_setform)
importFrom(curl,handle_setheaders)
importFrom(curl,handle_setopt)
importFrom(curl,multi_add)
importFrom(curl,multi_cancel)
importFrom(curl,multi_list)
importFrom(curl,multi_run)
importFrom(curl,new_handle)
importFrom(curl,new_pool)
importFrom(curl,parse_headers)
importFrom(httpcode,http_code)
importFrom(lifecycle,deprecated)
