agents <- c(
  "Mozilla/5.0 (Windows NT 5.1; pap-CW; rv:1.9.1.20) Gecko/2012-08-24 13:44:55 Firefox/3.6.5",
  "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_9) AppleWebKit/5330 (KHTML, like Gecko) Chrome/57.0.841.0 Safari/5330",
  "Opera/9.91.(Windows NT 4.0; tr-CY) Presto/2.9.179 Version/10.00",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_6_4) AppleWebKit/5320 (KHTML, like Gecko) Chrome/47.0.869.0 Safari/5320",
  "Opera/9.68.(Windows NT 6.1; aa-ER) Presto/2.9.164 Version/10.00",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_10_8 rv:2.0; af-ZA) AppleWebKit/533.14.7 (KHTML, like Gecko) Version/5.0.1 Safari/533.14.7",
  "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_0) AppleWebKit/5312 (KHTML, like Gecko) Chrome/15.0.847.0 Safari/5312",
  "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/5321 (KHTML, like Gecko) Chrome/37.0.815.0 Safari/5321",
  "Opera/8.83.(Windows NT 5.2; cy-GB) Presto/2.9.184 Version/11.00",
  "Opera/9.26.(X11; Linux x86_64; ss-ZA) Presto/2.9.183 Version/12.00",
  "Mozilla/5.0 (compatible; MSIE 5.0; Windows NT 6.2; Trident/3.0)",
  "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_11_0) AppleWebKit/5361 (KHTML, like Gecko) Chrome/49.0.815.0 Safari/5361",
  "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/5360 (KHTML, like Gecko) Chrome/29.0.881.0 Safari/5360",
  "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_7 rv:2.0; as-IN) AppleWebKit/535.32.2 (KHTML, like Gecko) Version/4.1 Safari/535.32.2",
  "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/5340 (KHTML, like Gecko) Chrome/21.0.828.0 Safari/5340",
  "Opera/9.89.(X11; Linux i686; eo-US) Presto/2.9.166 Version/12.00",
  "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/5361 (KHTML, like Gecko) Chrome/49.0.891.0 Safari/5361",
  "Mozilla/5.0 (X11; Linux x86_64; rv:1.9.6.20) Gecko/2015-10-03 20:33:41 Firefox/3.6.1",
  "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_9_9 rv:3.0; lv-LV) AppleWebKit/535.50.1 (KHTML, like Gecko) Version/4.0.1 Safari/535.50.1",
  "Opera/9.63.(X11; Linux x86_64; ts-ZA) Presto/2.9.163 Version/12.00",
  "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 6.2; Trident/3.0)",
  "Mozilla/5.0 (X11; Linux i686) AppleWebKit/5360 (KHTML, like Gecko) Chrome/39.0.807.0 Safari/5360",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_9_9 rv:4.0; ayc-PE) AppleWebKit/535.39.4 (KHTML, like Gecko) Version/4.1 Safari/535.39.4",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_8_4; rv:1.9.6.20) Gecko/2018-05-18 05:50:56 Firefox/3.8",
  "Opera/8.29.(X11; Linux x86_64; el-CY) Presto/2.9.173 Version/10.00",
  "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/5.1)",
  "Opera/9.38.(X11; Linux x86_64; km-KH) Presto/2.9.188 Version/10.00",
  "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_2) AppleWebKit/5330 (KHTML, like Gecko) Chrome/52.0.879.0 Safari/5330",
  "Opera/8.73.(X11; Linux x86_64; ayc-PE) Presto/2.9.188 Version/11.00",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_10_1) AppleWebKit/5332 (KHTML, like Gecko) Chrome/47.0.847.0 Safari/5332",
  "Mozilla/5.0 (X11; Linux i686) AppleWebKit/5340 (KHTML, like Gecko) Chrome/36.0.876.0 Safari/5340",
  "Opera/8.14.(X11; Linux x86_64; ik-CA) Presto/2.9.184 Version/11.00",
  "Mozilla/5.0 (compatible; MSIE 6.0; Windows NT 5.1; Trident/5.0)",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_9_4; rv:1.9.6.20) Gecko/2019-03-05 11:42:54 Firefox/3.6.16",
  "Mozilla/5.0 (compatible; MSIE 7.0; Windows 98; Trident/3.0)",
  "Mozilla/5.0 (compatible; MSIE 5.0; Windows 95; Trident/5.0)",
  "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_12_3; rv:1.9.3.20) Gecko/2015-07-31 13:06:37 Firefox/12.0",
  "Mozilla/5.0 (Windows NT 5.0) AppleWebKit/5330 (KHTML, like Gecko) Chrome/32.0.858.0 Safari/5330",
  "Opera/8.32.(Windows NT 4.0; ak-GH) Presto/2.9.163 Version/12.00",
  "Mozilla/5.0 (Windows NT 4.0; ga-IE; rv:1.9.1.20) Gecko/2017-12-21 17:38:03 Firefox/3.8",
  "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; mr-IN) AppleWebKit/534.22.6 (KHTML, like Gecko) Version/3.0.5 Mobile/8B114 Safari/6534.22.6",
  "Mozilla/5.0 (Macintosh; PPC Mac OS X 10_11_4) AppleWebKit/5331 (KHTML, like Gecko) Chrome/28.0.875.0 Safari/5331",
  "Opera/8.11.(X11; Linux x86_64; mg-MG) Presto/2.9.176 Version/10.00",
  "Mozilla/5.0 (X11; Linux i686) AppleWebKit/5352 (KHTML, like Gecko) Chrome/14.0.823.0 Safari/5352",
  "Opera/8.93.(X11; Linux i686; mai-IN) Presto/2.9.177 Version/11.00",
  "Mozilla/5.0 (Windows; U; Windows NT 5.0) AppleWebKit/534.39.5 (KHTML, like Gecko) Version/5.1 Safari/534.39.5",
  "Opera/8.68.(Windows NT 5.2; ayc-PE) Presto/2.9.162 Version/12.00",
  "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/3.1)",
  "Opera/8.79.(X11; Linux x86_64; nl-AW) Presto/2.9.174 Version/12.00",
  "Opera/9.89.(Windows 98; zh-CN) Presto/2.9.180 Version/11.00"
)
