#' progress bars
#'
#' @name progress
#' @details pass `httr::progress()` to `progress` param in [HttpClient],
#' which pulls out relevant info to pass down to \pkg{curl}
#'
#' if file sizes known you get progress bar; if file sizes not
#' known you get bytes downloaded
#'
#' See the README for examples
NULL
